export * from '@drupal-js-sdk/core';

class DrupalError extends Error {
  constructor(code, message) {
    super(message);
    this.code = code;
    Object.defineProperty(this, "message", {
      enumerable: true,
      value: message
    });
  }
  getErrorCode() {
    return this.code;
  }
  toString() {
    return `DrupalError: ${this.code} ${this.message}`;
  }
}
DrupalError.UNDEFINED = -1;
DrupalError.INTERNAL_SERVER_ERROR = 1;
DrupalError.MISSING_IMPLEMENTATION_ERROR = 42;
DrupalError.CONNECTION_FAILED = 100;
DrupalError.OBJECT_NOT_FOUND = 101;
DrupalError.INVALID_QUERY = 102;
DrupalError.INVALID_CLASS_NAME = 103;
DrupalError.MISSING_OBJECT_ID = 104;
DrupalError.INVALID_KEY_NAME = 105;
DrupalError.INVALID_POINTER = 106;
DrupalError.INVALID_JSON = 107;
DrupalError.COMMAND_UNAVAILABLE = 108;
DrupalError.NOT_INITIALIZED = 109;
DrupalError.INCORRECT_TYPE = 111;
DrupalError.INVALID_CHANNEL_NAME = 112;
DrupalError.PUSH_MISCONFIGURED = 115;
DrupalError.OBJECT_TOO_LARGE = 116;
DrupalError.OPERATION_FORBIDDEN = 119;
DrupalError.CACHE_MISS = 120;
DrupalError.INVALID_NESTED_KEY = 121;
DrupalError.INVALID_FILE_NAME = 122;
DrupalError.INVALID_ACL = 123;
DrupalError.TIMEOUT = 124;
DrupalError.INVALID_EMAIL_ADDRESS = 125;
DrupalError.MISSING_CONTENT_TYPE = 126;
DrupalError.MISSING_CONTENT_LENGTH = 127;
DrupalError.INVALID_CONTENT_LENGTH = 128;
DrupalError.FILE_TOO_LARGE = 129;
DrupalError.FILE_SAVE_ERROR = 130;
DrupalError.DUPLICATE_VALUE = 137;
DrupalError.INVALID_ROLE_NAME = 139;
DrupalError.EXCEEDED_QUOTA = 140;
DrupalError.SCRIPT_FAILED = 141;
DrupalError.VALIDATION_ERROR = 142;
DrupalError.INVALID_IMAGE_DATA = 143;
DrupalError.UNSAVED_FILE_ERROR = 151;
DrupalError.INVALID_PUSH_TIME_ERROR = 152;
DrupalError.FILE_DELETE_ERROR = 153;
DrupalError.FILE_DELETE_UNNAMED_ERROR = 161;
DrupalError.REQUEST_LIMIT_EXCEEDED = 155;
DrupalError.DUPLICATE_REQUEST = 159;
DrupalError.INVALID_EVENT_NAME = 160;
DrupalError.INVALID_VALUE = 162;
DrupalError.USERNAME_MISSING = 200;
DrupalError.PASSWORD_MISSING = 201;
DrupalError.USERNAME_TAKEN = 202;
DrupalError.EMAIL_TAKEN = 203;
DrupalError.EMAIL_MISSING = 204;
DrupalError.EMAIL_NOT_FOUND = 205;
DrupalError.SESSION_MISSING = 206;
DrupalError.MUST_CREATE_USER_THROUGH_SIGNUP = 207;
DrupalError.ACCOUNT_ALREADY_LINKED = 208;
DrupalError.INVALID_SESSION_TOKEN = 209;
DrupalError.MFA_ERROR = 210;
DrupalError.MFA_TOKEN_REQUIRED = 211;
DrupalError.LINKED_ID_MISSING = 250;
DrupalError.INVALID_LINKED_SESSION = 251;
DrupalError.UNSUPPORTED_SERVICE = 252;
DrupalError.INVALID_SCHEMA_OPERATION = 255;
DrupalError.AGGREGATE_ERROR = 600;
DrupalError.FILE_READ_ERROR = 601;
DrupalError.X_DOMAIN_REQUEST = 602;
DrupalError.STORAGE_IN_MEMORY_FAIL = 700;
DrupalError.STORAGE_IN_WEB_FAIL = 701;

class StorageInMemory {
  constructor() {
    this.data = {};
    if (this.isAvailable() === false) {
      throw new DrupalError(DrupalError.STORAGE_IN_MEMORY_FAIL, "Storage is not available.");
    }
  }
  getString(keyName) {
    return this.getItem(keyName);
  }
  setString(keyName, keyValue) {
    this.setItem(keyName, keyValue);
  }
  isAvailable() {
    try {
      const test_string = "__test_data_used_for_checking_storage__";
      this.setString(test_string, test_string);
      const isAvailable = this.getString(test_string) === test_string;
      this.removeItem(test_string);
      return isAvailable;
    } catch (e) {
      return false;
    }
  }
  getItem(keyName) {
    const value = this.data[keyName];
    return value === void 0 ? null : this.data[keyName];
  }
  setItem(keyName, keyValue) {
    this.data[keyName] = keyValue;
  }
  removeItem(keyName) {
    delete this.data[keyName];
  }
  clear() {
    this.set({});
  }
  get() {
    return this.data;
  }
  set(data) {
    this.data = data;
  }
}

class StorageInWeb {
  constructor(getStorage = () => window.localStorage) {
    this.storage = getStorage();
    if (this.isAvailable() === false) {
      throw new DrupalError(DrupalError.STORAGE_IN_WEB_FAIL, "Storage is not available.");
    }
  }
  getString(keyName) {
    return this.storage.getItem(keyName);
  }
  setString(keyName, keyValue) {
    this.storage.setItem(keyName, keyValue);
  }
  isAvailable() {
    try {
      const test_string = "__test_data_used_for_checking_strorage__";
      this.setString(test_string, test_string);
      const isAvailable = this.getString(test_string) === test_string;
      this.removeItem(test_string);
      return isAvailable;
    } catch (e) {
      return false;
    }
  }
  getItem(keyName) {
    const value = this.getString(keyName);
    return value !== null ? JSON.parse(value) : null;
  }
  setItem(keyName, keyValue) {
    const encodedValue = JSON.stringify(keyValue);
    this.setString(keyName, encodedValue);
  }
  removeItem(keyName) {
    return this.storage.removeItem(keyName);
  }
  clear() {
    this.storage.clear();
  }
  get() {
    throw new Error('Not possible to use "get" in WebStorage context.');
  }
  set() {
    throw new Error('Not possible to use "set" in WebStorage context.');
  }
}

export { DrupalError, StorageInMemory, StorageInWeb };
